# -*- coding: utf-8 -*-


import convert


def as_unicode(obj, encoding=convert.LOCALE):
    """
    Representing any object to unicode string.

    :param obj: any object
    :type encoding: str
    :param encoding: codec for encoding unicode strings
                     (locale.getpreferredencoding() by default)

    :rtype: unicode
    :return: any object as unicode string
    """

    return convert.convert(obj, encoding)


def as_str(obj, encoding=convert.LOCALE):
    """
    Representing any object to string.

    :param obj: any object
    :type encoding: str
    :param encoding: codec for encoding unicode strings
                     (locale.getpreferredencoding() by default)

    :rtype: str
    :return: any object as string
    """

    return as_unicode(obj, encoding).encode(encoding)



# -*- coding: utf-8 -*-


import collections
import locale


LOCALE = locale.getpreferredencoding()


def unimapping(arg):
    """
    Mapping object to unicode string.

    :type arg: collections.Mapping
    :param arg: mapping object

    :rtype: unicode
    :return: mapping object as unicode string
    """

    if not isinstance(arg, collections.Mapping):
        raise TypeError(
            'expected collections.Mapping, {} received'.format(type(arg).__name__)
        )

    result = []
    for i in arg.items():
        result.append(u': '.join(map(convert, i)))

    return u'{{{}}}'.format(u', '.join(result))


def uniiterable(arg):
    """
    Iterable object to unicode string.

    :type arg: collections.Iterable
    :param arg: iterable object

    :rtype: unicode
    :return: iterable object as unicode string
    """

    if not isinstance(arg, collections.Iterable):
        raise TypeError(
            'expected collections.Iterable, {} received'.format(type(arg).__name__)
        )

    templates = {
        list: u'[{}]',
        tuple: u'({})'
    }

    result = []
    for i in arg:
        result.append(convert(i))

    return templates.get(type(arg), templates[tuple]).format(u', '.join(result))


def convert(obj, encoding=LOCALE):
    """
    Covert any object to unicode string.

    :param obj: any object
    :type encoding: str
    :param encoding: codec for encoding unicode strings
                     (locale.getpreferredencoding() by default)

    :rtype: unicode
    :return: any object as unicode string
    """

    func = lambda x: u"u'{}'".format(x)

    if isinstance(obj, unicode):
        # skip if condition, because unicode is a iterable type
        pass
    elif isinstance(obj, str):
        func = lambda x: u"'{}'".format(x.decode(encoding))
    elif isinstance(obj, (type(None), int, float)):
        func = lambda x: unicode(x)
    elif isinstance(obj, collections.Mapping):
        func = unimapping
    elif isinstance(obj, collections.Iterable):
        func = uniiterable

    return func(obj)



# -*- coding: utf-8 -*-


import re
import os

from setuptools import setup


DIR = os.path.dirname(__file__)


with open(os.path.join(DIR, 'unilog', '__init__.py')) as f:
    version = re.search(r'__version__\s+=\s+[\'\"]+(.*)[\'\"]+', f.read()).group(1)


setup(
    name='unilog',
    version=version,
    packages=['unilog'],
    url='https://github.com/oleg-golovanov/unilog',
    license='MIT',
    author='Oleg Golovanov',
    author_email='golovanov.ov@gmail.com',
    description='Representing complex object as unicode or simple string.',
    zip_safe=False,
    platforms='any',
    long_description=open(os.path.join(DIR, 'README.rst')).read(),
    classifiers=[
        'Development Status :: 4 - Beta',
        'Intended Audience :: Developers',
        'Operating System :: OS Independent',
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: Implementation :: CPython',
        'Topic :: Software Development :: Libraries :: Python Modules'
    ]
)
